'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.API = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; /*
                                                                                                                                                                                                                                                                               *  Handle methods for creating APIs
                                                                                                                                                                                                                                                                               */


var _camo = require('@uh-sem-blee/camo');

var _camo2 = _interopRequireDefault(_camo);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 *  Some generic errors
 */
var ENO_MODEL = { code: 404, message: 'No model set on API' };
var EINVALID_ID = { code: 400, message: 'Invalid ID' };

class API {
  constructor(mighty, config) {
    var _this2 = this;

    this.globalConfig = mighty.config;
    this.app = mighty.app;
    this.config = config;
    this.model = null;
    this.dbClient = (0, _camo.getClient)();

    if (this.config.model) {
      this.model = mighty.models[this.config.model.name] = this.config.model;
      console.log('Setup model: ' + this.config.model.name + ' in [' + this.config.data_space + ']');
    }

    this.handlers = {
      get: function get(req, res, next) {
        // example, need to consolidate this
        var _this = _this2;
        if (!_this.model) {
          API.handleError(res, ENO_MODEL);
          return next();
        }

        var skip = 0;
        var limit = 50;
        var sortDir = 1;
        var sort = { 'create_time': sortDir };

        var conditions = _utils2.default.convertParamToObject(req.params.conditions);
        var fields = _utils2.default.convertParamToObject(req.params.fields);
        req.params.not = _utils2.default.convertParamToObject(req.params.not);
        var search = _utils2.default.convertParamToObject(req.params.search);

        if (req.params.sortDir) {
          if (req.params.sort.charAt(0) === '[' || req.params.sort.charAt(0) === '{') {
            try {
              sortDir = JSON.parse(req.params.sortDir);
            } catch (err) {
              API.handleError(res, { code: 500, message: 'Invalid sortDir parameter' });
            }
          } else {
            sortDir = parseInt(req.params.sortDir, 10);
          }
        }

        if (req.params.sort) {
          if (req.params.sort.charAt(0) === '[' || req.params.sort.charAt(0) === '{') {
            try {
              sort = JSON.parse(req.params.sort);
            } catch (err) {
              API.handleError(res, { code: 500, message: 'Invalid sort parameter' });
            }
          } else {
            sort = {};
            sort[req.params.sort] = sortDir;
          }
        }

        if (req.params._id) {
          conditions._id = req.params._id;
        }

        if (conditions._id) {
          if (conditions._id instanceof Array) {
            var Ids = { $in: [] };
            for (var i = 0; i < conditions._id.length; i++) {
              try {
                Ids['$in'].push(_this.dbClient.toNativeId(conditions._id[i]));
              } catch (e) {
                return API.handleError(res, { code: 400, message: e.message });
              }
            }
            conditions._id = Ids;
          } else if (_typeof(conditions._id) === 'object') {
            for (var i in conditions._id) {
              if (conditions._id[i] instanceof Array) {
                for (var j = 0; j < conditions._id[i].length; j++) {
                  try {
                    conditions._id[i][j] = _this.dbClient.toNativeId(conditions._id[i][j]);
                  } catch (e) {
                    return API.handleError(res, { code: 400, message: e.message });
                  }
                }
              } else {
                try {
                  conditions._id[i] = _this.dbClient.toNativeId(conditions._id[i]);
                } catch (e) {
                  return API.handleError(res, { code: 400, message: 'Invalid ID' });
                }
              }
            }
          } else {
            try {
              conditions._id = _this.dbClient.toNativeId(conditions._id);
            } catch (e) {
              return API.handleError(res, { code: 400, message: 'Invalid ID' });
            }
          }
        }

        var method = 'find';

        if (conditions._id) {
          method = 'findOne';
        }

        var args = {};
        if (limit) args.limit = limit;
        if (skip) args.skip = skip;
        if (sort) args.sort = sort;

        var q = _this.model[method](conditions);
        if (fields) {
          q.select(fields);
        }
        q.then(function (results) {
          res.send(200, results);
          return next();
        }).catch(function (err) {
          API.handleError(res, { code: 500, message: err });
          return next();
        });
      },
      post: function post(req, res, next) {
        var _this = _this2;
        if (!_this.model) {
          API.handleError(res, ENO_MODEL);
          return next();
        }

        var m = _this.model.create(req.params);
        m.save().then(function (result) {
          if (req.route.config && req.route.config.afterConditions) {
            _utils2.default.handleAfterConditions(result, req.route.config.afterConditions, req).then(function (result) {
              res.send(200, result);
              return next();
            }).catch(function (err) {
              API.handleError(res, { code: 500, message: err });
              return next();
            });
          } else {
            res.send(200, result);
            return next();
          }
        }).catch(function (err) {
          API.handleError(res, { code: 500, message: err });
          return next();
        });
      },
      put: function put(req, res, next) {
        var _this = _this2;
        if (!_this.model) {
          API.handleError(res, ENO_MODEL);
          return next();
        }
        if (!req.params._id) {
          API.handleError(res, { code: 404, message: 'Missing _id' });
          return next();
        }

        _this2.model.findOneAndUpdate(req.params.conditions, req.params, { new: true }).then(function (result) {
          if (result === null) {
            API.handleError(res, { code: 404, message: _this.config.model.name + ' not found!' });
            return next();
          }

          res.send(200, result);
          return next();
        }).catch(function (err) {
          API.handleError(res, { code: 500, message: err });
          return next();
        });
      },
      del: function del(req, res, next) {
        var _this = _this2;
        if (!_this.model) {
          API.handleError(res, ENO_MODEL);
          return next();
        }

        if (!req.params._id) {
          API.handleError(res, { code: 404, message: 'Missing _id' });
          return next();
        }

        _this2.model.findOneAndDelete(req.params._id).then(function (result) {
          if (result === null) {
            API.handleError(res, { code: 404, message: _this.config.model.name + ' not found!' });
            return next();
          }

          res.send(200, result);
          return next();
        }).catch(function (err) {
          API.handleError(res, { code: 500, message: err });
          return next();
        });
      }
    };
  }

  /*
   * Handle an API error
   *
   * @param {object} res - The result
   * @param {object} error - The error object
   * @param {number} error.code - The status code to be returned.
   * @param {string} error.message - The message to be returned.
   */
  static handleError(res, error) {
    return res.json(error.code, error);
  }

  static createRoute(context, path, method) {
    var verbose = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;

    var app = context.app;
    var config = Object.assign({}, context.config);
    app[method]({
      path: path,
      version: config.version,
      name: config.name,
      config: config.api[method]
    }, context.handlers[method]);
    if (verbose) {
      console.log('Setup route: [' + method.toUpperCase() + '] ' + path);
    }

    if (method === 'get') {
      path += '/:_id';
      app[method]({
        path: path,
        version: config.version,
        name: config.name,
        config: config.api[method]
      }, context.handlers[method]);
      if (verbose) {
        console.log('Setup route: [' + method.toUpperCase() + '] ' + path);
      }
    }
  }

  /*
   * Load an API with a configuration
   *
   * @param {object} config - The configuration for the API
   */
  loadAPI() {
    var config = this.config;
    var app = this.app;
    try {
      if (config.api) {
        for (var method in config.api) {
          // setup admin routes if necessary, overrides 'enabled' for standard CRUD
          if (config.api[method].admin) {
            if (!config.api[method].admin_prefix) {
              config.api[method].admin_prefix = 'admin';
            }
            var _path = '/' + config.api[method].admin_prefix + '/' + config.data_space;
            if (config.route_space) {
              _path = '/admin' + config.route_space;
            }
            if (method === 'put' || method === 'del') {
              _path += '/:_id';
            }
            API.createRoute(this, _path, method);
          }

          if (!config.api[method].enabled) continue;

          var path = '/' + config.data_space;
          if (config.route_space) {
            path = config.route_space;
          }
          if (method === 'put' || method === 'del') {
            path += '/:_id';
          }
          API.createRoute(this, path, method);
        }
      }

      if (config.routes) {
        for (var _path2 in config.routes) {
          var _method = config.routes[_path2].method.toLowerCase();
          app[_method]({
            path: _path2,
            version: config.version,
            name: config.name,
            config: config.routes[_path2]
          }, config.routes[_path2].controller);
          console.log('Setup route: [' + _method.toUpperCase() + '] ' + _path2);
        }
      }
    } catch (err) {
      console.log(err);
      return null;
    }

    return this;
  }
}
exports.API = API;