'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _restify = require('restify');

var _restify2 = _interopRequireDefault(_restify);

var _package = require('../../package.json');

var _package2 = _interopRequireDefault(_package);

var _minimist = require('minimist');

var _minimist2 = _interopRequireDefault(_minimist);

var _socket = require('socket.io');

var _socket2 = _interopRequireDefault(_socket);

var _socket3 = require('socket.io-client');

var _socket4 = _interopRequireDefault(_socket3);

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _camo = require('@uh-sem-blee/camo');

var _camo2 = _interopRequireDefault(_camo);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Mighty extends _events2.default {
  constructor(config) {
    super();
    var base_dir = __dirname;
    this.argv = (0, _minimist2.default)(process.argv);
    var _this = this;
    this.baseConfig = require('./config.js').default;
    if (config) {
      this.config = config;
    } else {
      this.config = this.baseConfig;
    }

    if (!this.argv.e) {
      this.argv.e = 'development';
    }
    if (!this.argv.p) {
      this.argv.p = process.cwd();
    }

    this.app = null;
    this.environment = this.argv.e;
    this.cwd = this.argv.p;
    this.modules = {};
    this.models = {};
    _camo2.default.connect(this.config.database.url).then(function (db) {
      _this.db = db;
      _this.emit('database-loaded', db);
    });

    this.app = _restify2.default.createServer({ name: this.config.application_name, version: _package2.default.version });

    // CORS Stuff
    this.app.use(_restify2.default.CORS({
      origins: this.config.CORS_ORIGINS ? this.config.CORS_ORIGINS : ['*'],
      credentials: true
    }));
    _restify2.default.CORS.ALLOW_HEADERS.push('accept');
    _restify2.default.CORS.ALLOW_HEADERS.push('sid');
    _restify2.default.CORS.ALLOW_HEADERS.push('lang');
    _restify2.default.CORS.ALLOW_HEADERS.push('origin');
    _restify2.default.CORS.ALLOW_HEADERS.push('withcredentials');
    _restify2.default.CORS.ALLOW_HEADERS.push('x-requested-with');
    _restify2.default.CORS.ALLOW_HEADERS.push('apikey');
    _restify2.default.CORS.ALLOW_HEADERS.push('access-token');
    _restify2.default.CORS.ALLOW_HEADERS.push('authentication-key');
    _restify2.default.CORS.ALLOW_HEADERS.push('session-id');
    _restify2.default.CORS.ALLOW_HEADERS.push('authentication');

    if (this.config.CORS_HEADERS) {
      for (var i = 0; i < this.config.CORS_HEADERS.length; i++) {
        _restify2.default.CORS.ALLOW_HEADERS.push(this.config.CORS_HEADERS[i]);
      }
    }

    this.app.on("MethodNotAllowed", function (request, response) {
      if (request.method.toUpperCase() === "OPTIONS") {
        // Send the CORS headers for preflight requests (annoying hack)
        if (_restify2.default.CORS.origins.indexOf('*') === -1) {
          if (_restify2.default.CORS.origins.indexOf(request.headers.origin) === -1) {
            response.send(new _restify2.default.MethodNotAllowedError());
          }
        }
        response.header("Access-Control-Allow-Credentials", true);
        response.header("Access-Control-Allow-Headers", _restify2.default.CORS.ALLOW_HEADERS.join(", "));
        response.header("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE, OPTIONS");
        response.header("Access-Control-Allow-Origin", request.headers.origin);
        response.header("Access-Control-Max-Age", 0);
        response.header("Content-type", "text/plain charset=UTF-8");
        response.header("Content-length", 0);

        response.send(204);
      } else {
        response.send(new _restify2.default.MethodNotAllowedError());
      }
    });

    this.app.use(_restify2.default.fullResponse());
    this.app.use(_restify2.default.acceptParser(this.app.acceptable));
    this.app.use(_restify2.default.bodyParser({ overrideParams: true }));
    this.app.use(_restify2.default.queryParser());
    this.app.use(_restify2.default.gzipResponse());

    this.core = {
      api: require(base_dir + '/api').API,
      connector: require(base_dir + '/connector').Connector,
      utils: require(base_dir + '/utils')
    };
    this.app.use(this.core.utils.default.beforeConditions);

    return _this.start();
  }

  loadModules(folder) {
    var _this = this;
    var modules = _fs2.default.readdirSync(folder);
    modules.forEach(function (filename) {
      var filepath = folder + '/' + filename;
      if (_fs2.default.statSync(filepath).isDirectory()) {
        try {
          _fs2.default.accessSync(filepath + '/index.js', _fs2.default.F_OK);
          var m = require(filepath);
          _this.modules[filename] = new m(_this);
          if (_this.modules[filename].config) {
            _this.modules[filename].api = new _this.core.api(_this, _this.modules[filename].config).loadAPI();
            _this.models[_this.modules[filename].config.data_space] = _this.modules[filename].api.model;
          }
        } catch (err) {
          return console.log('Module: ' + filepath + ' had a problem:\
          ' + err.message + '\
          ' + err.stack + '\
          ');
        }
      }
    });
  }

  start() {
    var _this2 = this;

    return new Promise(function (resolve, reject) {
      var _this = _this2;

      _this.on('database-loaded', function (db) {
        _this.connector = new _this.core.connector(_this).start();

        if (_this.config.connection.type === 'hub') {
          _this.loadModules(_this.baseConfig.module_folder);
        } else if (_this.config.connection.type === 'spoke') {
          (function () {
            var master_host = _this.config.connection.hub_host;
            if (!master_host) {
              if (_this.environment === 'production') {
                master_host = 'https://app.mightyspace.xyz'; // production container
              } else if (_this.environment === 'testing') {
                master_host = 'https://testing.mightyspace.xyz'; // testing container
              } else {
                master_host = 'http://mighty-core:3459'; // developer container
              }
            }
            // connect to the "master_host"
            _this.master_socket = (0, _socket4.default)(master_host);
            console.log('Attempting to connect to Master Host: ' + master_host);
            _this.master_socket.on('error', function (err) {
              console.log(err);
            });
            _this.master_socket.on('connect', function () {
              console.log('Connected to Master Host: ' + master_host);
            });
          })();
        }

        _this.app.listen(_this.config.api_port, function () {
          console.log('%s listening at %s in %s mode', _this.config.application_name, 'http://' + _this.config.application_host + ':' + _this.config.api_port, _this.environment);

          if (_this.config.connection.type === 'spoke') {
            return resolve(_this);
          }

          // check for the first user.
          // if we have one, then APIs should continue as normal
          var User = _this.models['User'];

          User.findOne({}).then(function (user) {
            if (!user) {
              console.log('Waiting for first user');
              _this.first_user = true;
            }
            return resolve(_this);
          }).catch(function (err) {
            console.log(err);
            return reject(err);
            process.exit(1);
          });
        });
      });
    });
  }

  getModel(data_space) {
    return this.models[data_space];
  }
}
exports.default = Mighty;