'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _config = require('../organizations/config.js');

var _config2 = _interopRequireDefault(_config);

var _camo = require('@uh-sem-blee/camo');

var _utils = require('../../utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class User extends _camo.Document {
  constructor() {
    super();
    this.schema({
      name: { type: String },
      email: { type: String, required: true, unique: true },
      authentication_key: { type: String, select: false, default: null },
      secret: { type: String, select: false },
      password: { type: String, select: false },
      reset_code: { type: String, select: false },
      organizations: [_config2.default.model],
      default_organization: _config2.default.model,
      organization_profiles: [{
        permissions: [{ type: String }]
      }],
      avatar_url: { type: String },
      active: { type: Boolean, default: false },
      verified: { type: Boolean, default: false },
      verification_code: { type: String, select: false, default: _utils2.default.generateKey },
      last_login: { type: Date, default: new Date() },
      role: { type: String, default: 'user', choices: ['user', 'admin'] }
    });
  }
}

var config = {
  module_name: 'Users',
  data_space: 'users',
  route_space: '/users',
  version: '1.0.0',
  dependencies: {
    // 'instrmnt.other_module': 'x.x.x' -- use the data_space
  },
  model: User,
  api: {
    get: {
      enabled: false,
      admin: true
      // jsapi: false,
      // jsapi_authentication: false,
      // socket: false,
      // beforeConditions: {},
      // afterConditions: {}
    },
    post: {
      enabled: false,
      admin: true
    },
    put: {
      enabled: true,
      authentication: true,
      owner: true,
      admin: true
    },
    del: {
      enabled: false,
      admin: true
    }
  }
};

exports.default = config;