'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _validator = require('validator');

var _validator2 = _interopRequireDefault(_validator);

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _auth = require('../auth');

var _auth2 = _interopRequireDefault(_auth);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Login {
  constructor(app) {
    var _this2 = this;

    this.app = app;
    this.handleLogin = function (req, res, next) {
      var _this = _this2;
      var User = _config2.default.model;
      if (!_validator2.default.isEmail(req.params.email + '')) {
        return _this.app.core.api.handleError(res, { code: 404, message: 'Invalid email' });
      }
      if (!_validator2.default.isAlphanumeric(req.params.password + '')) {
        return _this.app.core.api.handleError(res, { code: 404, message: 'Invalid password' });
      }

      User.findOne({ email: req.params.email }, { populate: ['email', 'authentication_key', 'secret', 'password'] }).then(function (user) {
        if (!user) {
          return _this.app.core.api.handleError(res, { code: 404, message: 'Could not find user with that email and password' });
        }

        _auth2.default.generatePassword(user.secret, req.params.password).then(function (encodedPassword) {
          if (encodedPassword !== user.password) {
            return _this.app.core.api.handleError(res, { code: 404, message: 'Could not find user with that email and password' });
          }

          var out = user.toJSON();

          delete out.password;
          delete out.secret;
          delete out.verification_code;

          res.json(out);
        }).catch(function (err) {
          return _this.app.core.api.handleError(res, { code: 500, message: err.message });
        });
      }).catch(function (err) {
        return _this.app.core.api.handleError(res, { code: 500, message: err.message });
      });
    };

    this.handleForgotPassword = function (req, res, next) {
      var _this = _this2;
      var User = _config2.default.model;
      if (!_validator2.default.isEmail(req.params.email + '')) {
        return _this.app.core.api.handleError(res, { code: 404, message: 'Invalid email' });
      }

      User.findOne({ email: req.params.email }).select('+reset_code').then(function (user) {
        if (!user) {
          return _this.app.core.api.handleError(res, { code: 404, message: 'Could not find user with that email and password' });
        }

        // send to notifier service
      });
    };

    this.handleResetPassword = function (req, res, next) {
      var _this = _this2;
      var User = _config2.default.model;

      User.findOne({ reset_code: req.params.reset_code }).then(function (user) {
        if (!user) {
          return _this.app.core.api.handleError(res, { code: 404, message: 'Could not find user with that email and password' });
        }

        if (!_validator2.default.isAlphanumeric(req.params.password + '')) {
          return _this.app.core.api.handleError(res, { code: 404, message: 'Invalid password' });
        }
        // send to notifier service
      });
    };
  }
}exports.default = Login;
;