'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _validator = require('validator');

var _validator2 = _interopRequireDefault(_validator);

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _auth = require('../auth');

var _auth2 = _interopRequireDefault(_auth);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class UserRegisterModule {
  constructor(app) {
    var _this2 = this;

    this.app = app;

    this.createUser = function (params) {
      var User = _config2.default.model;
      var _this = _this2;

      return new Promise(function (resolve, reject) {
        params.email = params.email.toLowerCase();
        params.avatar_url = '//www.gravatar.com/avatar/' + _crypto2.default.createHash('md5').update(params.email).digest("hex");
        params.secret = _crypto2.default.randomBytes(32).toString('hex');
        params.authentication_key = _auth2.default.generateAuthenticationKey(params.email, params.secret);
        _auth2.default.generatePassword(params.secret, params.password).then(function (encodedPassword) {
          params.password = encodedPassword;
          var m = User.create(params);
          m.save().then(function (user) {
            user.save().then(function (user) {
              resolve(user);
            }).catch(function (err) {
              reject(err);
            });
          }).catch(function (err) {
            reject(err);
          });
        }).catch(function (err) {
          reject(err);
        });
      });
    };

    this.handleRegister = function (req, res, next) {
      var _this = _this2;
      if (!_validator2.default.isEmail(req.params.email + '')) {
        return _this.app.core.api.handleError(res, { code: 404, message: 'Invalid email' });
      }
      if (!_validator2.default.isAlphanumeric(req.params.password + '')) {
        return _this.app.core.api.handleError(res, { code: 404, message: 'Invalid password' });
      }

      _this.createUser(req.params).then(function (user) {
        if (_this.app.first_user) {
          delete _this.app.first_user;
        }
        var out = user.toJSON();

        delete out.password;
        delete out.secret;
        delete out.verification_code;

        return res.json(out);
      }).catch(function (err) {
        var message = err.message;
        if (err.code === 11000) {
          message = 'User is already registered. If you forgot your password use the link to reset your password';
        }
        _this.app.core.api.handleError(res, { code: 500, message: message });
      });
    };
  }
}exports.default = UserRegisterModule;
;