'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

var _api = require('./api');

var _api2 = _interopRequireDefault(_api);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 *  Utility methods that may come in handy in projects
 */
class Utils {
  constructor() {}

  static convertParamToObject(param) {
    if (param && typeof param === 'string') {
      try {
        param = JSON.parse(param);
      } catch (e) {
        return {};
      }
    } else if (!param) {
      return {};
    }

    return param;
  }

  /**
    * Generate a pretty unique prefix
    *
    */
  static generatePrefix() {
    return Math.floor(Math.random() * 1000000000).toString(36);
  }

  /**
    *  Generate a unique key that can be used for authorization, password reset, etc
    */
  static generateKey() {
    var salt = new Date().getTime();
    return require('crypto').createHash('sha256').update((0, _uuid2.default)()).update(salt.toString()).digest('hex');
  }

  /**
   *  handleAfterConditions is an API method used to create adjustments to this result or other model documents after the call has been made.
   *
   */
  static handleAfterConditions(result, conditions, req) {
    var _this = this;

    return new Promise(function (resolve, reject) {
      if (!conditions || !result) {
        return resolve(result);
      }

      if (conditions instanceof Function) {
        return conditions(req, result).then(function (result) {
          return resolve(result);
        });
      }

      for (var action in conditions) {
        var condition = conditions[action];
        var model = void 0;
        if (!condition.request) {
          model = result;
        } else if (condition.request) {
          model = req[condition.request];
        }

        switch (action) {
          case '$push':
            if (!(model[condition.field] instanceof Array)) {
              model[condition.field] = [];
            }
            model[condition.field].push(result[condition.value]);
            break;
          case '$set':
            model[condition.field] = result[condition.value];
            break;
          case '$delete':
            delete model[condition.field];
            break;
        }

        if (condition.save) {
          model.save();
        }
      }

      return resolve(result);
    });
  }

  /**
   *  handleBeforeConditions is an API method used to inject conditions for your API that override any request. This can be useful for querying objects that are only 'active' or similar situations.
   *
   */
  static beforeConditions(req, res, next) {
    if (!req.route.config || !req.route.config.beforeConditions) {
      return next();
    }

    if (req.route.config.beforeConditions instanceof Function) {
      return req.route.config.beforeConditions(req).then(function () {
        return next();
      }).catch(function (err) {
        return _api2.default.handleError(res, { code: err.code || 500, message: err.message || 'No error message' });
      });
    }

    for (var action in req.route.config.beforeConditions) {
      var condition = req.route.config.beforeConditions[action];
      switch (action) {
        case '$add':
          req.params.conditions = Utils.convertParamToObject(req.params.conditions);
          for (var key in condition) {
            req.params.conditions[key] = condition[key];
          }
          break;
        case '$remove':
          req.params.conditions = Utils.convertParamToObject(req.params.conditions);
          for (var _key in condition) {
            delete req.params.conditions[_key];
          }
          break;
      }
    }

    return next();
  }
}
exports.default = Utils;